import fs_implementation from 'node:fs';
import process from 'node:process';
import path from 'node:path';

function sync_impl$g$2(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$2(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$2 = {sync_impl: sync_impl$g$2, async_impl: async_impl$g$2};

function sync_impl$f$2(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$2(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$2 = {sync_impl: sync_impl$f$2, async_impl: async_impl$f$2};

function sync_impl$e$2(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$2(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$4 = {sync_impl: sync_impl$e$2, async_impl: async_impl$e$2};

function sync_impl$d$2(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$2(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$2 = {sync_impl: sync_impl$d$2, async_impl: async_impl$d$2};

function sync_impl$c$2(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$2(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$2 = {sync_impl: sync_impl$c$2, async_impl: async_impl$c$2};

function sync_impl$b$2(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$2(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$2 = {sync_impl: sync_impl$b$2, async_impl: async_impl$b$2};

function sync_impl$a$2(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$2(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$2 = {sync_impl: sync_impl$a$2, async_impl: async_impl$a$2};

function sync_impl$9$2(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$2(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$2 = {sync_impl: sync_impl$9$2, async_impl: async_impl$9$2};

function sync_impl$8$2(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$2(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$2 = {sync_impl: sync_impl$8$2, async_impl: async_impl$8$2};

function sync_impl$7$2(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$2(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$2 = {sync_impl: sync_impl$7$2, async_impl: async_impl$7$2};

function sync_impl$6$2(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$2(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$2 = {sync_impl: sync_impl$6$2, async_impl: async_impl$6$2};

function sync_impl$5$2(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$2(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$2 = {sync_impl: sync_impl$5$2, async_impl: async_impl$5$2};

function sync_impl$4$2(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$2(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$2 = {sync_impl: sync_impl$4$2, async_impl: async_impl$4$2};

function sync_impl$3$2(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$2(fs, path) {
	return await fs.promises.stat(path)
}

var stat$2 = {sync_impl: sync_impl$3$2, async_impl: async_impl$3$2};

function sync_impl$2$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$2 = {sync_impl: sync_impl$2$2, async_impl: async_impl$2$2};

function sync_impl$1$2(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$2(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$2 = {sync_impl: sync_impl$1$2, async_impl: async_impl$1$2};

function sync_impl$k(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$k(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$2 = {sync_impl: sync_impl$k, async_impl: async_impl$k};

/* warning: this file was automatically generated */

var methods$2 = {
    chmod: chmod$2,
    chown: chown$2,
    copyFile: copyFile$4,
    lstat: lstat$2,
    mkdir: mkdir$2,
    openDirectory: openDirectory$2,
    openFile: openFile$2,
    readFile: readFile$2,
    readdir: readdir$2,
    readlink: readlink$2,
    realpath: realpath$2,
    rename: rename$2,
    rmdir: rmdir$2,
    stat: stat$2,
    symlink: symlink$2,
    unlink: unlink$2,
    writeFile: writeFile$2,
};

function createFSObject$2({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$2) {
		const method = methods$2[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function sync_impl$g$1(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$1(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$1 = {sync_impl: sync_impl$g$1, async_impl: async_impl$g$1};

function sync_impl$f$1(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$1(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$1 = {sync_impl: sync_impl$f$1, async_impl: async_impl$f$1};

function sync_impl$e$1(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$1(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$3 = {sync_impl: sync_impl$e$1, async_impl: async_impl$e$1};

function sync_impl$d$1(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$1(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$1 = {sync_impl: sync_impl$d$1, async_impl: async_impl$d$1};

function sync_impl$c$1(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$1(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$1 = {sync_impl: sync_impl$c$1, async_impl: async_impl$c$1};

function sync_impl$b$1(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$1(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$1 = {sync_impl: sync_impl$b$1, async_impl: async_impl$b$1};

function sync_impl$a$1(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$1(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$1 = {sync_impl: sync_impl$a$1, async_impl: async_impl$a$1};

function sync_impl$9$1(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$1(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$1 = {sync_impl: sync_impl$9$1, async_impl: async_impl$9$1};

function sync_impl$8$1(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$1(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$1 = {sync_impl: sync_impl$8$1, async_impl: async_impl$8$1};

function sync_impl$7$1(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$1(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$1 = {sync_impl: sync_impl$7$1, async_impl: async_impl$7$1};

function sync_impl$6$1(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$1(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$1 = {sync_impl: sync_impl$6$1, async_impl: async_impl$6$1};

function sync_impl$5$1(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$1(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$1 = {sync_impl: sync_impl$5$1, async_impl: async_impl$5$1};

function sync_impl$4$1(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$1(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$1 = {sync_impl: sync_impl$4$1, async_impl: async_impl$4$1};

function sync_impl$3$1(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$1(fs, path) {
	return await fs.promises.stat(path)
}

var stat$1 = {sync_impl: sync_impl$3$1, async_impl: async_impl$3$1};

function sync_impl$2$1(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$1(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$1 = {sync_impl: sync_impl$2$1, async_impl: async_impl$2$1};

function sync_impl$1$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$1 = {sync_impl: sync_impl$1$1, async_impl: async_impl$1$1};

function sync_impl$h$1(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$1(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$1 = {sync_impl: sync_impl$h$1, async_impl: async_impl$h$1};

/* warning: this file was automatically generated */

var methods$1 = {
    chmod: chmod$1,
    chown: chown$1,
    copyFile: copyFile$3,
    lstat: lstat$1,
    mkdir: mkdir$1,
    openDirectory: openDirectory$1,
    openFile: openFile$1,
    readFile: readFile$1,
    readdir: readdir$1,
    readlink: readlink$1,
    realpath: realpath$1,
    rename: rename$1,
    rmdir: rmdir$1,
    stat: stat$1,
    symlink: symlink$1,
    unlink: unlink$1,
    writeFile: writeFile$1,
};

function createFSObject$1({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$1) {
		const method = methods$1[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function tryStat$1(fs_object, path, fn = "lstat") {
	try {
		return fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

function sync_impl$j(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = tryStat$1(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = tryStat$1(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

async function tryStat(fs_object, path, fn = "lstat") {
	try {
		return await fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

async function async_impl$j(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = await tryStat(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = await tryStat(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

const async_fs$2 = createFSObject$1({sync: false});
const sync_fs$2 = createFSObject$1({sync: true});

function nodeFsGetPathType(...args) {
	return async_impl$j(async_fs$2, ...args)
}

nodeFsGetPathType.sync = function(...args) {
	return sync_impl$j(sync_fs$2, ...args)
};

function sync_impl$g(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod = {sync_impl: sync_impl$g, async_impl: async_impl$g};

function sync_impl$f(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown = {sync_impl: sync_impl$f, async_impl: async_impl$f};

function sync_impl$e(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$2 = {sync_impl: sync_impl$e, async_impl: async_impl$e};

function sync_impl$d(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat = {sync_impl: sync_impl$d, async_impl: async_impl$d};

function sync_impl$c(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir = {sync_impl: sync_impl$c, async_impl: async_impl$c};

function sync_impl$b(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory = {sync_impl: sync_impl$b, async_impl: async_impl$b};

function sync_impl$a(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile = {sync_impl: sync_impl$a, async_impl: async_impl$a};

function sync_impl$9(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile = {sync_impl: sync_impl$9, async_impl: async_impl$9};

function sync_impl$8(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir = {sync_impl: sync_impl$8, async_impl: async_impl$8};

function sync_impl$7(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink = {sync_impl: sync_impl$7, async_impl: async_impl$7};

function sync_impl$6(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath = {sync_impl: sync_impl$6, async_impl: async_impl$6};

function sync_impl$5(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename = {sync_impl: sync_impl$5, async_impl: async_impl$5};

function sync_impl$4(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir = {sync_impl: sync_impl$4, async_impl: async_impl$4};

function sync_impl$3(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3(fs, path) {
	return await fs.promises.stat(path)
}

var stat = {sync_impl: sync_impl$3, async_impl: async_impl$3};

function sync_impl$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink = {sync_impl: sync_impl$2, async_impl: async_impl$2};

function sync_impl$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink = {sync_impl: sync_impl$1, async_impl: async_impl$1};

function sync_impl$h(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile = {sync_impl: sync_impl$h, async_impl: async_impl$h};

/* warning: this file was automatically generated */

var methods = {
    chmod,
    chown,
    copyFile: copyFile$2,
    lstat,
    mkdir,
    openDirectory,
    openFile,
    readFile,
    readdir,
    readlink,
    realpath,
    rename,
    rmdir,
    stat,
    symlink,
    unlink,
    writeFile,
};

function createFSObject({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods) {
		const method = methods[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function scandir$1(fs_object, root_dir, relative_entry_dir, options) {
	const entries = fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.callback === "function") {
				options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = () => {
			if (type !== "dir") return

			scandir$1(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse) {
			recurse();
		}

		handle_current_entry();

		if (!options.reverse) {
			recurse();
		}
	}
}

function sync_impl$i(fs_object, root_dir, {
	callback = null,
	reverse = false
} = {}) {
	let entries = [];
	const options = {callback, reverse, entries};
	const resolved_root_path = fs_object.realpath(root_dir);

	scandir$1(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

async function scandir(fs_object, root_dir, relative_entry_dir, options) {
	const entries = await fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = await fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = async () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.callback === "function") {
				await options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = async () => {
			if (type !== "dir") return

			await scandir(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse) {
			await recurse();
		}

		await handle_current_entry();

		if (!options.reverse) {
			await recurse();
		}
	}
}

async function async_impl$i(fs_object, root_dir, {
	callback = null,
	reverse = false
} = {}) {
	let entries = [];
	const options = {callback, reverse, entries};
	const resolved_root_path = await fs_object.realpath(root_dir);

	await scandir(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

const async_fs$1 = createFSObject({sync: false});
const sync_fs$1 = createFSObject({sync: true});

function nodeFsScandir(root_dir, options = {}) {
	return async_impl$i(async_fs$1, root_dir, options)
}

nodeFsScandir.sync = function(root_dir, options = {}) {
	return sync_impl$i(sync_fs$1, root_dir, options)
};

function copySymbolicLink$1(fs_object, src, dest) {
	const link = fs_object.readlink(src);

	fs_object.symlink(link, dest);
}

function copyFile$1(fs_object, src, dest) {
	fs_object.copyFile(src, dest);
}

function copyDirectory$1(fs_object, src, dest) {
	fs_object.mkdir(dest, {
		recursive: true
	});

	nodeFsScandir.sync(src, {
		callback({type, relative_path, absolute_path}) {
			const entry_src = absolute_path;
			const entry_dest = path.join(dest, relative_path);

			const args = [fs_object, entry_src, entry_dest];

			if (type === "link") {
				copySymbolicLink$1(...args);
			} else if (type === "dir") {
				copyDirectory$1(...args);
			} else {
				copyFile$1(...args);
			}
		}
	});
}

const copy_map$1 = {
	"link->dir": copySymbolicLink$1,
	"link->file": copySymbolicLink$1,
	"link->broken": copySymbolicLink$1,
	"file": copyFile$1,
	"dir": copyDirectory$1
};

function sync_impl(fs_object, src, dest) {
	const path_type = nodeFsGetPathType.sync(src);

	if (!(path_type in copy_map$1)) {
		throw new Error(`Don't know how to copy path of type '${path_type}'.`)
	}

	const copy_fn = copy_map$1[path_type];

	return copy_fn(fs_object, src, dest)
}

async function copySymbolicLink(fs_object, src, dest) {
	const link = await fs_object.readlink(src);

	await fs_object.symlink(link, dest);
}

async function copyFile(fs_object, src, dest) {
	await fs_object.copyFile(src, dest);
}

async function copyDirectory(fs_object, src, dest) {
	await fs_object.mkdir(dest, {
		recursive: true
	});

	await nodeFsScandir(src, {
		async callback({type, relative_path, absolute_path}) {
			const entry_src = absolute_path;
			const entry_dest = path.join(dest, relative_path);

			const args = [fs_object, entry_src, entry_dest];

			if (type === "link") {
				await copySymbolicLink(...args);
			} else if (type === "dir") {
				await copyDirectory(...args);
			} else {
				await copyFile(...args);
			}
		}
	});
}

const copy_map = {
	"link->dir": copySymbolicLink,
	"link->file": copySymbolicLink,
	"link->broken": copySymbolicLink,
	"file": copyFile,
	"dir": copyDirectory
};

async function async_impl(fs_object, src, dest) {
	const path_type = await nodeFsGetPathType(src);

	if (!(path_type in copy_map)) {
		throw new Error(`Don't know how to copy path of type '${path_type}'.`)
	}

	const copy_fn = copy_map[path_type];

	return await copy_fn(fs_object, src, dest)
}

const async_fs = createFSObject$2({sync: false});
const sync_fs = createFSObject$2({sync: true});

function nodeFsCopy(src, dest) {
	return async_impl(async_fs, src, dest)
}

nodeFsCopy.sync = function(src, dest) {
	return sync_impl(sync_fs, src, dest)
};

export { nodeFsCopy as default };
